@@ -20,8 +20,7 @@
 # Some bits taken from quodlibet mpris plugin by <christoph.reiter@gmx.at>
 
 
-from __future__ import print_function
-
+from configparser import ConfigParser
 import os
 import sys
 import re
@@ -46,11 +45,6 @@ try:
 except ImportError:
     mutagen = None
 
-try:
-    import configparser
-except ImportError:
-    import ConfigParser as configparser
-
 try:
     import gi
     gi.require_version('Notify', '0.7')
@@ -576,11 +570,7 @@ class MPDWrapper(object):
             if not any([song_url.startswith(prefix) for prefix in urlhandlers]):
                 song_url = os.path.join(self._params['music_dir'], song_url)
             self._metadata['xesam:url'] = song_url
-            try:
-                cover = self.find_cover(song_url)
-            except mutagen.MutagenError as e:
-                logger.error("Can't extract covers from %r: %r" % (song_url, e))
-                cover = None
+            cover = self.find_cover(song_url)
             if cover:
                 self._metadata['mpris:artUrl'] = cover
 
@@ -638,8 +628,13 @@ class MPDWrapper(object):
                     self._temp_cover.close()
 
             # Search for embedded cover art
+            song = None
             if mutagen and os.path.exists(song_path):
-                song = mutagen.File(song_path)
+                try:
+                    song = mutagen.File(song_path)
+                except mutagen.MutagenError as e:
+                    logger.error("Can't extract covers from %r: %r" % (song_path, e))
+            if song is not None:
                 if song.tags:
                     # present but null for some file types
                     for tag in song.tags.keys():
@@ -671,9 +666,8 @@ class MPDWrapper(object):
                             self._temp_song_url = song_url
                             return self._create_temp_cover(pic)
 
-
             # Look in song directory for common album cover files
-            if os.path.exists(song_dir):
+            if os.path.exists(song_dir) and os.path.isdir(song_dir):
                 for f in os.listdir(song_dir):
                     if self._params['cover_regex'].match(f):
                         return 'file://' + os.path.join(song_dir, f)
@@ -712,14 +706,17 @@ class MPDWrapper(object):
         old_position = self._position
         old_time = self._time
         self._currentsong = self.currentsong()
-        self._status = new_status = self.status()
+        new_status = self.status()
         self._time = new_time = int(time.time())
-        logger.debug("_update_properties: current song = %r" % self._currentsong)
-        logger.debug("_update_properties: current status = %r" % self._status)
 
         if not new_status:
+            logger.debug("_update_properties: failed to get new status")
             return
 
+        self._status = new_status
+        logger.debug("_update_properties: current song = %r" % self._currentsong)
+        logger.debug("_update_properties: current status = %r" % self._status)
+
         if 'elapsed' in new_status:
             new_position = float(new_status['elapsed'])
         elif 'time' in new_status:
@@ -882,9 +879,7 @@ class MPDWrapper(object):
     def __getattr__(self, attr):
         if attr[0] == "_":
             raise AttributeError(attr)
-        def fn(*a, **kw):
-            return self.call(attr, *a, **kw)
-        return fn
+        return lambda *a, **kw: self.call(attr, *a, **kw)
 
     def call(self, command, *args):
         fn = getattr(self.client, command)
@@ -905,37 +900,57 @@ class NotifyWrapper(object):
 
     def __init__(self, params):
         self._notification = None
+        self._enabled = True
 
-        if not params["notify"]:
-            return
+        if params["notify"]:
+            self._notification = self._bootstrap_notifications()
+            if not self._notification:
+                logger.error("No notification service provider could be found; disabling notifications")
+        else:
+            self._enabled = False
 
+    def _bootstrap_notifications(self):
+        # Check if someone is providing the notification service
         bus = dbus.SessionBus()
         try:
             bus.get_name_owner("org.freedesktop.Notifications")
         except dbus.exceptions.DBusException:
-            logger.error("No service handling org.freedesktop.Notifications; disabling notifications")
-            return
+            return None
 
+        notif = None
+
+        # Bootstrap whatever notifications system we are using
         if using_gi_notify:
             logger.debug("Initializing GObject.Notify")
             if Notify.init(identity):
-                self._notification = Notify.Notification()
-                self._notification.set_hint("desktop-entry", GLib.Variant("s", "mpdris2"))
-                self._notification.set_hint("transient", GLib.Variant("b", True))
+                notif = Notify.Notification()
+                notif.set_hint("desktop-entry", GLib.Variant("s", "mpdris2"))
+                notif.set_hint("transient", GLib.Variant("b", True))
             else:
                 logger.error("Failed to init libnotify; disabling notifications")
-                self._notification = None
         elif using_old_notify:
             logger.debug("Initializing old pynotify")
             if pynotify.init(identity):
-                self._notification = pynotify.Notification("", "", "")
-                self._notification.set_hint("desktop-entry", "mpdris2")
-                self._notification.set_hint("transient", True)
+                notif = pynotify.Notification("", "", "")
+                notif.set_hint("desktop-entry", "mpdris2")
+                notif.set_hint("transient", True)
             else:
                 logger.error("Failed to init libnotify; disabling notifications")
-                self._notification = None
+
+        return notif
 
     def notify(self, title, body, uri=''):
+        if not self._enabled:
+            return
+        
+        # If we did not yet manage to get a notification service,
+        # try again
+        if not self._notification:
+            logger.info('Retrying to acquire a notification service provider...')
+            self._notification = self._bootstrap_notifications()
+            if self._notification:
+                logger.info('Notification service provider acquired!')
+        
         if self._notification:
             try:
                 self._notification.set_urgency(params['notify_urgency'])
@@ -1289,6 +1304,7 @@ Usage: %(progname)s [OPTION]...
          --music-dir=PATH   Set the music library path
 
      -d, --debug            Run in debug mode
+     -j, --use-journal      Log to systemd journal instead of stderr
      -v, --version          mpDris2 version
 
 Environment variables MPD_HOST and MPD_PORT can be used.
@@ -1301,17 +1317,20 @@ if __name__ == '__main__':
     gettext.bindtextdomain('mpDris2', '@datadir@/locale')
     gettext.textdomain('mpDris2')
 
-    log_format = '%(asctime)s %(module)s %(levelname)s: %(message)s'
+    log_format_stderr = '%(asctime)s %(module)s %(levelname)s: %(message)s'
+
+    log_journal = False
     log_level = logging.INFO
     config_file = None
     music_dir = None
 
     # Parse command line
     try:
-        (opts, args) = getopt.getopt(sys.argv[1:], 'c:dh:p:v',
+        (opts, args) = getopt.getopt(sys.argv[1:], 'c:dh:jp:v',
                                      ['help', 'bus-name=', 'config=',
                                       'debug', 'host=', 'music-dir=',
-                                      'path=', 'port=', 'version'])
+                                      'use-journal', 'path=', 'port=',
+                                      'version'])
     except getopt.GetoptError as ex:
         (msg, opt) = ex.args
         print("%s: %s" % (sys.argv[0], msg), file=sys.stderr)
@@ -1331,6 +1350,8 @@ if __name__ == '__main__':
             log_level = logging.DEBUG
         elif opt in ['-h', '--host']:
             params['host'] = arg
+        elif opt in ['-j', '--use-journal']:
+            log_journal = True
         elif opt in ['-p', '--path', '--music-dir']:
             music_dir = arg
         elif opt in ['--port']:
@@ -1346,8 +1367,24 @@ if __name__ == '__main__':
         usage(params)
         sys.exit()
 
-    logging.basicConfig(format=log_format, level=log_level)
     logger = logging.getLogger('mpDris2')
+    logger.propagate = False
+    logger.setLevel(log_level)
+
+    # Attempt to configure systemd journal logging, if enabled
+    if log_journal:
+        try:
+            from systemd.journal import JournalHandler
+            log_handler = JournalHandler(SYSLOG_IDENTIFIER='mpDris2')
+        except ImportError:
+            log_journal = False
+
+    # Log to stderr if journal logging was not enabled, or if setup failed
+    if not log_journal:
+        log_handler = logging.StreamHandler()
+        log_handler.setFormatter(logging.Formatter(log_format_stderr))
+
+    logger.addHandler(log_handler)
 
     # Pick up the server address (argv -> environment -> config)
     for arg in args[:2]:
@@ -1364,7 +1401,7 @@ if __name__ == '__main__':
             params['port'] = os.environ['MPD_PORT']
 
     # Read configuration
-    config = configparser.SafeConfigParser()
+    config = ConfigParser()
     if config_file:
         with open(config_file) as fh:
             config.read(config_file)
@@ -1383,6 +1420,8 @@ if __name__ == '__main__':
     if '@' in params['host']:
         params['password'], params['host'] = params['host'].rsplit('@', 1)
 
+    params['host'] = os.path.expanduser(params['host'])
+
     for p in ['mmkeys', 'notify']:
         if config.has_option('Bling', p):
             params[p] = config.getboolean('Bling', p)
@@ -1403,6 +1442,7 @@ if __name__ == '__main__':
         if not re.match('^[0-9A-Za-z+.-]+://', music_dir):
             music_dir = 'file://' + music_dir
         if music_dir.startswith('file://'):
+            music_dir = music_dir[:7] + os.path.expanduser(music_dir[7:])
             if not os.path.exists(music_dir[7:]):
                 logger.error('Music library path %s does not exist!' % music_dir)
         # Non-local URLs can still be useful to MPRIS clients, so accept them.
